(*
 *  beware ircd, Internet Relay Chat server, rndpass.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)


unit
  rndpass;

{
simple password garble unit, not compatible with unix passwords :(
}

interface

const
  KK=17;
  JJ=10;
  R1=13;
  R2=9;
type
  int=smallint;

procedure randominit(seed:longint);
function byterandom:byte;
function garble(s:string):string;

var
  p1,p2:longint;
  randbuffer:array[0..kk-1] of longint;

implementation

function rotl(l,rr:longint):longint;
var
  b:longint;
begin
  if rr = 0 then begin
    rotl := rr;
    exit
  end;
  b := l shr (32-rr);
  l := (l shl rr) or b;
  rotl := l;
end;

procedure randominit(seed:longint);
var
  i:longint;
begin
  if seed = 0 then dec(seed);
  for i := 0 to kk-1 do begin
    seed := seed xor (seed shl 13); seed := seed xor (seed shr 17); seed := seed xor (seed shl 5);
    randbuffer[i] := seed;
  end;
  p1 := 0;  p2 := JJ;
  for i := 0 to 9 do byterandom;
end;

function byterandom;
var
  x:longint;
begin
  randbuffer[p1] := rotl(randbuffer[p2], R1) + rotl(randbuffer[p1], R2);
  x := randbuffer[p1];
  byterandom := x shr 24
end;

function garble(s:string):string;
const
  outchar:array[0..63] of char='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.\';
  outlen=13;
  xx=17;
var
  bitbuf,bitbuf2:array[0..511] of byte;
  bytebuf:array[0..63] of byte;
  s2:string;
  l,m,n:longint;
  a,b,c:int;
begin
  fillchar(bitbuf,sizeof(bitbuf),0);
  fillchar(bitbuf2,sizeof(bitbuf2),0);
  if s = '' then begin
    garble := '';
    exit
  end;
  for a := 1 to length(s) do begin
    {rotate left}
    fillchar(bytebuf,sizeof(bytebuf),0);
    for b := 0 to 63 do for c := 0 to 7 do begin
      bytebuf[b] := bytebuf[b] or (bitbuf[b shl 3 or c] shl c);
    end;
    l := 0;
    m := 1;
    for b := 1 to length(s) do begin
      n := byte(s[(a+b) mod length(s)+1]);
      inc(l,m*n);
      m := m * xx;
      if m > 1 shl 24 then m := 1
    end;
    randominit(l);
    for b := 0 to 63 do inc(bytebuf[b],byterandom);
    for b := 0 to 63 do for c := 0 to 7 do begin
      bitbuf[b shl 3 or c] := (bytebuf[b] shr c) and 1
    end;
    c := byterandom;
    move(bitbuf[0],bitbuf2,c);
    move(bitbuf[c],bitbuf[0],512-c);
    move(bitbuf2,bitbuf[512-c],c);
  end;
  s2 := '';
  for a := 0 to outlen-1 do begin
    c := 0;
    for b := 0 to 5 do c := c or (bitbuf[6*a+b] shl b);
    s2 := s2 + outchar[c]
  end;
  garble := s2;
end;

end.
